/*
    Copyright 2013 Kevin Timmerman

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// GlowWithTheShow.cpp
//

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <string>

using namespace std;


struct T_CMD {
	char *name;
	unsigned len;
	uint8_t *data;
};

uint8_t cmd_off[] = { 0x90, 0x60 };
uint8_t cmd_blue[] = { 0x90, 0x61 };
uint8_t cmd_green[] = { 0x90, 0x62 };
uint8_t cmd_cyan[] = { 0x90, 0x63 };
uint8_t cmd_red[] = { 0x90, 0x64 };
uint8_t cmd_magenta[] = { 0x90, 0x65 };
uint8_t cmd_yellow[] = { 0x90, 0x66 };
uint8_t cmd_white[] = { 0x90, 0x67 };
uint8_t cmd_off_r[] = { 0x90, 0x68 };
uint8_t cmd_blue_r[] = { 0x90, 0x69 };
uint8_t cmd_green_r[] = { 0x90, 0x6A };
uint8_t cmd_cyan_r[] = { 0x90, 0x6B };
uint8_t cmd_red_r[] = { 0x90, 0x6C };
uint8_t cmd_magenta_r[] = { 0x90, 0x6D };
uint8_t cmd_yellow_r[] = { 0x90, 0x6E };
uint8_t cmd_white_r[] = { 0x90, 0x6F };

T_CMD cmds[] = {
	{ "Off",		2,	 cmd_off },
	{ "Blue",		2,	 cmd_blue },
	{ "Green",		2,	 cmd_green },
	{ "Cyan",		2,	 cmd_cyan },
	{ "Red",		2,	 cmd_red },
	{ "Magenta",	2,	 cmd_magenta },
	{ "Yellow",		2,	 cmd_yellow },
	{ "White",		2,	 cmd_white },
	{ "Off_R",		2,	 cmd_off_r },
	{ "Blue_R",		2,	 cmd_blue_r },
	{ "Green_R",	2,	 cmd_green_r },
	{ "Cyan_R",		2,	 cmd_cyan_r },
	{ "Red_R",		2,	 cmd_red_r },
	{ "Magenta_R",	2,	 cmd_magenta_r },
	{ "Yellow_R",	2,	 cmd_yellow_r },
	{ "White_R",	2,	 cmd_white_r },
};

void crc(uint8_t *d, unsigned len)
{
    uint8_t crc = 0;
    while(len--) {
        crc ^= *d++;
        unsigned n = 8; do crc = (crc & 1) ? (crc >> 1) ^ 0x8C : crc >> 1; while(--n);
    }
    *d = crc;
	//cout << "CRC: " << hex << (unsigned)crc << endl << dec;
}

int main(int argc, char* argv[])
{
	T_CMD *c = cmds;

	while(c->name) {
		//cout << endl << c->name << endl;

		fstream out(string(c->name) + ".ict", fstream::out);

		uint8_t *pkt = new uint8_t[c->len + 1];
		memcpy(pkt, c->data, c->len);
		crc(pkt, c->len);

		vector<int> tv;

		int tt = 0;
		for(unsigned n = 0; n < c->len + 1; ++n) {
			for(unsigned b = 0; b < 10; ++b) {
				int t;
				if(b == 0) {
					t = 417;
				} else if(b == 9) {
					t = -417;
				} else {
					t = (pkt[n] & (1 << (b - 1))) ? -417 : 417;
				}
				if(t < 0) {
					if(tt <= 0) {
						tt += t;
					} else {
						tv.push_back(tt);
						tt = t;
					}
				} else {
					if(tt >= 0) {
						tt += t;
					} else {
						tv.push_back(tt);
						tt = t;
					}
				}
			}
		}
		tv.push_back(tt);

		delete[] pkt;

		out << "irscope 0" << endl;
		out << "carrier_frequency 38005" << endl;
		out << "sample_count " << tv.size() << endl;
		for(unsigned n = 0; n < tv.size(); ++n) {
			if(tv[n] >= 0) out << '+';
			out << tv[n] << endl;
		}

		out.close();

		++c;
	}

	return 0;
}
